const RippleDefaults = {
    color: 'rgba(0, 0, 0, 0.1)',
    enterDuration: '400ms',
    enterTimingFunction: 'linear',
    leaveDuration: '1000ms',
    leaveTimingFunction: 'ease-in-out',
    centered: false,
    abortAnimation: true,
    triggerOnParent: false,
};
class Ripple {
    constructor(element, options) {
        this.animationElement = undefined;
        this.element = element;
        this.options = { ...RippleDefaults, ...options };
        this.injectContainerStyle();
        if (this.options.triggerOnParent) {
            this.triggerElement = this.element.parentElement || this.element;
        }
        else {
            this.triggerElement = this.element;
        }
        this.element.classList.add('ripple-container');
        this.triggerElement.addEventListener('pointerdown', this.startAnimation.bind(this));
    }
    destroy() {
        this.element.classList.remove('ripple-container');
        this.triggerElement.removeEventListener('pointerdown', this.startAnimation);
    }
    startAnimation(event) {
        event.stopImmediatePropagation();
        if (this.animationElement && this.options.abortAnimation) {
            this.abortAnimation();
        }
        this.createAnimationElement(event);
        const stop = () => {
            this.triggerElement.removeEventListener('pointerup', stop);
            this.triggerElement.removeEventListener('pointerleave', stop);
            if (this.animationElement) {
                this.stopAnimation(this.animationElement);
            }
        };
        this.triggerElement.addEventListener('pointerup', stop);
        this.triggerElement.addEventListener('pointerleave', stop);
    }
    stopAnimation(animationElement) {
        animationElement.addEventListener('transitionend', (event) => {
            if (event.propertyName === 'opacity') {
                animationElement.remove();
            }
        });
        animationElement.style.opacity = '0';
    }
    abortAnimation() {
        if (this.animationElement) {
            this.animationElement.remove();
        }
    }
    createAnimationElement(event) {
        this.animationElement = document.createElement('div');
        this.animationElement.classList.add('ripple');
        this.element.appendChild(this.animationElement);
        const enterTransition = `${this.options.enterDuration} ${this.options.enterTimingFunction} 0s`;
        const leaveTransition = `${this.options.leaveDuration} ${this.options.leaveTimingFunction} 0s`;
        const style = this.animationElement.style;
        style.position = 'absolute';
        style.width = style.maxWidth = style.paddingBottom = '0%';
        style.height = '0';
        style.transform = 'translate(-50%, -50%)';
        style.borderRadius = '50%';
        style.backgroundColor = `${this.options.color}`;
        style.opacity = '1';
        style.pointerEvents = 'none';
        style.transition =
            `width ${enterTransition},` +
                `max-width ${enterTransition},` +
                `padding-bottom ${enterTransition},` +
                `opacity ${leaveTransition}`;
        const { clientX: x, clientY: y } = event;
        const { width, height, top, left, right, bottom, } = this.element.getBoundingClientRect();
        let targetWidth;
        if (this.options.centered) {
            style.top = '50%';
            style.left = '50%';
            targetWidth = 2 * Math.sqrt(50 ** 2 + 50 ** 2);
        }
        else {
            const topPercent = ((y - top) / height) * 100;
            const bottomPercent = ((bottom - y) / height) * 100;
            const leftPercent = ((x - left) / width) * 100;
            const rightPercent = ((right - x) / width) * 100;
            style.top = `${topPercent}%`;
            style.left = `${leftPercent}%`;
            targetWidth =
                2 *
                    Math.sqrt(Math.max(leftPercent, rightPercent) ** 2 +
                        Math.max(topPercent, bottomPercent) ** 2);
        }
        // force reflow
        this.animationElement.offsetWidth;
        style.width = style.maxWidth = style.paddingBottom = `${targetWidth}%`;
    }
    injectContainerStyle() {
        if (document.head.querySelector('style.ripple-container') !== null) {
            return;
        }
        const style = document.createElement('style');
        style.classList.add('ripple-container');
        style.innerHTML = `
      .ripple-container {
        position: relative;
        overflow: hidden;
      }
    `;
        document.head.appendChild(style);
    }
}
function ripple(node, options = {}) {
    const ripple = new Ripple(node, options);
    return {
        destroy() {
            ripple.destroy();
        },
    };
}

export { ripple };
//# sourceMappingURL=index.esm.js.map
